@@ -185,19 +185,21 @@ impl ClusterSlotsService {
 #[cfg(test)]
 mod test {
     use super::*;
-    use crate::cluster_info::Node;
+    use crate::{cluster_info::Node, crds_value::CrdsValueLabel};
 
     #[test]
     pub fn test_update_lowest_slot() {
-        let node_info = Node::new_localhost_with_pubkey(&Pubkey::default());
+        let pubkey = Pubkey::new_unique();
+        let node_info = Node::new_localhost_with_pubkey(&pubkey);
         let cluster_info = ClusterInfo::new_with_invalid_keypair(node_info.info);
-        ClusterSlotsService::update_lowest_slot(&Pubkey::default(), 5, &cluster_info);
+        ClusterSlotsService::update_lowest_slot(&pubkey, 5, &cluster_info);
         cluster_info.flush_push_queue();
-        let lowest = cluster_info
-            .get_lowest_slot_for_node(&Pubkey::default(), None, |lowest_slot, _| {
-                lowest_slot.clone()
-            })
-            .unwrap();
+        let lowest = {
+            let label = CrdsValueLabel::LowestSlot(pubkey);
+            let gossip = cluster_info.gossip.read().unwrap();
+            let entry = gossip.crds.get(&label).unwrap();
+            entry.value.lowest_slot().unwrap().clone()
+        };
         assert_eq!(lowest.lowest, 5);
     }
 }
