@@ -11,15 +11,14 @@
 use crate::{
     cluster_info::CRDS_UNIQUE_PUBKEY_CAPACITY,
     contact_info::ContactInfo,
-    crds::{Crds, Cursor, VersionedCrdsValue},
-    crds_gossip::{get_stake, get_weight, CRDS_GOSSIP_DEFAULT_BLOOM_ITEMS},
+    crds::{Crds, Cursor},
+    crds_gossip::{get_stake, get_weight},
     crds_gossip_error::CrdsGossipError,
     crds_value::CrdsValue,
     weighted_shuffle::weighted_shuffle,
 };
 use bincode::serialized_size;
 use indexmap::map::IndexMap;
-use itertools::Itertools;
 use lru::LruCache;
 use rand::{seq::SliceRandom, Rng};
 use solana_runtime::bloom::{AtomicBloom, Bloom};
@@ -53,7 +52,10 @@ pub struct CrdsGossipPush {
     /// bool indicates it has been pruned.
     /// This cache represents a lagging view of which validators
     /// currently have this node in their `active_set`
-    received_cache: HashMap<Pubkey, HashMap<Pubkey, (bool, u64)>>,
+    received_cache: HashMap<
+        Pubkey, // origin/owner
+        HashMap</*gossip peer:*/ Pubkey, (/*pruned:*/ bool, /*timestamp:*/ u64)>,
+    >,
     last_pushed_to: LruCache<Pubkey, u64>,
     pub num_active: usize,
     pub push_fanout: usize,
@@ -102,67 +104,58 @@ impl CrdsGossipPush {
     ) -> Vec<Pubkey> {
         let origin_stake = stakes.get(origin).unwrap_or(&0);
         let self_stake = stakes.get(self_pubkey).unwrap_or(&0);
-        let cache = self.received_cache.get(origin);
-        if cache.is_none() {
-            return Vec::new();
-        }
-        let peers = cache.unwrap();
-
+        let peers = match self.received_cache.get_mut(origin) {
+            None => return Vec::default(),
+            Some(peers) => peers,
+        };
         let peer_stake_total: u64 = peers
             .iter()
-            .filter(|v| !(v.1).0)
-            .map(|v| stakes.get(v.0).unwrap_or(&0))
+            .filter(|(_, (pruned, _))| !pruned)
+            .filter_map(|(peer, _)| stakes.get(peer))
             .sum();
         let prune_stake_threshold = Self::prune_stake_threshold(*self_stake, *origin_stake);
         if peer_stake_total < prune_stake_threshold {
             return Vec::new();
         }
-
-        let staked_peers: Vec<(Pubkey, u64)> = peers
-            .iter()
-            .filter(|v| !(v.1).0)
-            .filter_map(|p| stakes.get(p.0).map(|s| (*p.0, *s)))
-            .filter(|(_, s)| *s > 0)
-            .collect();
-
-        let mut seed = [0; 32];
-        rand::thread_rng().fill(&mut seed[..]);
-        let shuffle = weighted_shuffle(
-            &staked_peers.iter().map(|(_, stake)| *stake).collect_vec(),
-            seed,
-        );
-
+        let shuffled_staked_peers = {
+            let peers: Vec<_> = peers
+                .iter()
+                .filter(|(_, (pruned, _))| !pruned)
+                .filter_map(|(peer, _)| Some((*peer, *stakes.get(peer)?)))
+                .filter(|(_, stake)| *stake > 0)
+                .collect();
+            let mut seed = [0; 32];
+            rand::thread_rng().fill(&mut seed[..]);
+            let weights: Vec<_> = peers.iter().map(|(_, stake)| *stake).collect();
+            weighted_shuffle(&weights, seed)
+                .into_iter()
+                .map(move |i| peers[i])
+        };
         let mut keep = HashSet::new();
         let mut peer_stake_sum = 0;
         keep.insert(*origin);
-        for next in shuffle {
-            let (next_peer, next_stake) = staked_peers[next];
-            if next_peer == *origin {
+        for (peer, stake) in shuffled_staked_peers {
+            if peer == *origin {
                 continue;
             }
-            keep.insert(next_peer);
-            peer_stake_sum += next_stake;
+            keep.insert(peer);
+            peer_stake_sum += stake;
             if peer_stake_sum >= prune_stake_threshold
                 && keep.len() >= CRDS_GOSSIP_PRUNE_MIN_INGRESS_NODES
             {
                 break;
             }
         }
-
-        let pruned_peers: Vec<Pubkey> = peers
+        for (peer, (pruned, _)) in peers.iter_mut() {
+            if !*pruned && !keep.contains(peer) {
+                *pruned = true;
+            }
+        }
+        peers
             .keys()
-            .filter(|p| !keep.contains(p))
-            .cloned()
-            .collect();
-        pruned_peers.iter().for_each(|p| {
-            self.received_cache
-                .get_mut(origin)
-                .unwrap()
-                .get_mut(p)
-                .unwrap()
-                .0 = true;
-        });
-        pruned_peers
+            .filter(|peer| !keep.contains(peer))
+            .copied()
+            .collect()
     }
 
     fn wallclock_window(&self, now: u64) -> impl RangeBounds<u64> {
@@ -176,7 +169,7 @@ impl CrdsGossipPush {
         from: &Pubkey,
         value: CrdsValue,
         now: u64,
-    ) -> Result<Option<VersionedCrdsValue>, CrdsGossipError> {
+    ) -> Result<(), CrdsGossipError> {
         self.num_total += 1;
         if !self.wallclock_window(now).contains(&value.wallclock()) {
             return Err(CrdsGossipError::PushMessageTimeout);
@@ -270,46 +263,48 @@ impl CrdsGossipPush {
         network_size: usize,
         ratio: usize,
     ) {
+        const BLOOM_FALSE_RATE: f64 = 0.1;
+        const BLOOM_MAX_BITS: usize = 1024 * 8 * 4;
+        #[cfg(debug_assertions)]
+        const MIN_NUM_BLOOM_ITEMS: usize = 512;
+        #[cfg(not(debug_assertions))]
+        const MIN_NUM_BLOOM_ITEMS: usize = CRDS_UNIQUE_PUBKEY_CAPACITY;
         let mut rng = rand::thread_rng();
         let need = Self::compute_need(self.num_active, self.active_set.len(), ratio);
         let mut new_items = HashMap::new();
-
-        let options: Vec<_> = self.push_options(
-            crds,
-            &self_id,
-            self_shred_version,
-            stakes,
-            gossip_validators,
-        );
-        if options.is_empty() {
+        let (weights, peers): (Vec<_>, Vec<_>) = self
+            .push_options(
+                crds,
+                &self_id,
+                self_shred_version,
+                stakes,
+                gossip_validators,
+            )
+            .into_iter()
+            .unzip();
+        if peers.is_empty() {
             return;
         }
-
-        let mut seed = [0; 32];
-        rng.fill(&mut seed[..]);
-        let mut shuffle = weighted_shuffle(
-            &options.iter().map(|weighted| weighted.0).collect_vec(),
-            seed,
-        )
-        .into_iter();
-
-        while new_items.len() < need {
-            match shuffle.next() {
-                Some(index) => {
-                    let item = options[index].1;
-                    if self.active_set.get(&item.id).is_some() {
-                        continue;
-                    }
-                    if new_items.get(&item.id).is_some() {
-                        continue;
-                    }
-                    let size = cmp::max(CRDS_GOSSIP_DEFAULT_BLOOM_ITEMS, network_size);
-                    let bloom: AtomicBloom<_> = Bloom::random(size, 0.1, 1024 * 8 * 4).into();
-                    bloom.add(&item.id);
-                    new_items.insert(item.id, bloom);
-                }
-                _ => break,
+        let num_bloom_items = MIN_NUM_BLOOM_ITEMS.max(network_size);
+        let shuffle = {
+            let mut seed = [0; 32];
+            rng.fill(&mut seed[..]);
+            weighted_shuffle(&weights, seed).into_iter()
+        };
+        for peer in shuffle.map(|i| peers[i].id) {
+            if new_items.len() >= need {
+                break;
             }
+            if self.active_set.contains_key(&peer) || new_items.contains_key(&peer) {
+                continue;
+            }
+            let bloom = AtomicBloom::from(Bloom::random(
+                num_bloom_items,
+                BLOOM_FALSE_RATE,
+                BLOOM_MAX_BITS,
+            ));
+            bloom.add(&peer);
+            new_items.insert(peer, bloom);
         }
         let mut keys: Vec<Pubkey> = self.active_set.keys().cloned().collect();
         keys.shuffle(&mut rng);
@@ -462,9 +457,9 @@ mod test {
         // push a new message
         assert_eq!(
             push.process_push_message(&mut crds, &Pubkey::default(), value.clone(), 0),
-            Ok(None)
+            Ok(())
         );
-        assert_eq!(crds.lookup(&label), Some(&value));
+        assert_eq!(crds.get(&label).unwrap().value, value);
 
         // push it again
         assert_matches!(
@@ -483,7 +478,7 @@ mod test {
         // push a new message
         assert_eq!(
             push.process_push_message(&mut crds, &Pubkey::default(), value, 0),
-            Ok(None)
+            Ok(())
         );
 
         // push an old version
@@ -527,19 +522,16 @@ mod test {
 
         // push a new message
         assert_eq!(
-            push.process_push_message(&mut crds, &Pubkey::default(), value_old.clone(), 0),
-            Ok(None)
+            push.process_push_message(&mut crds, &Pubkey::default(), value_old, 0),
+            Ok(())
         );
 
         // push an old version
         ci.wallclock = 1;
         let value = CrdsValue::new_unsigned(CrdsData::ContactInfo(ci));
         assert_eq!(
-            push.process_push_message(&mut crds, &Pubkey::default(), value, 0)
-                .unwrap()
-                .unwrap()
-                .value,
-            value_old
+            push.process_push_message(&mut crds, &Pubkey::default(), value, 0),
+            Ok(())
         );
     }
     #[test]
@@ -560,7 +552,7 @@ mod test {
             0,
         )));
 
-        assert_eq!(crds.insert(value1.clone(), now), Ok(None));
+        assert_eq!(crds.insert(value1.clone(), now), Ok(()));
         push.refresh_push_active_set(&crds, &HashMap::new(), None, &Pubkey::default(), 0, 1, 1);
 
         assert!(push.active_set.get(&value1.label().pubkey()).is_some());
@@ -569,7 +561,7 @@ mod test {
             0,
         )));
         assert!(push.active_set.get(&value2.label().pubkey()).is_none());
-        assert_eq!(crds.insert(value2.clone(), now), Ok(None));
+        assert_eq!(crds.insert(value2.clone(), now), Ok(()));
         for _ in 0..30 {
             push.refresh_push_active_set(&crds, &HashMap::new(), None, &Pubkey::default(), 0, 1, 1);
             if push.active_set.get(&value2.label().pubkey()).is_some() {
@@ -582,7 +574,7 @@ mod test {
             let value2 = CrdsValue::new_unsigned(CrdsData::ContactInfo(
                 ContactInfo::new_localhost(&solana_sdk::pubkey::new_rand(), 0),
             ));
-            assert_eq!(crds.insert(value2.clone(), now), Ok(None));
+            assert_eq!(crds.insert(value2.clone(), now), Ok(()));
         }
         push.refresh_push_active_set(&crds, &HashMap::new(), None, &Pubkey::default(), 0, 1, 1);
         assert_eq!(push.active_set.len(), push.num_active);
@@ -740,7 +732,7 @@ mod test {
             &solana_sdk::pubkey::new_rand(),
             0,
         )));
-        assert_eq!(crds.insert(peer.clone(), now), Ok(None));
+        assert_eq!(crds.insert(peer.clone(), now), Ok(()));
         push.refresh_push_active_set(&crds, &HashMap::new(), None, &Pubkey::default(), 0, 1, 1);
 
         let new_msg = CrdsValue::new_unsigned(CrdsData::ContactInfo(ContactInfo::new_localhost(
@@ -751,7 +743,7 @@ mod test {
         expected.insert(peer.label().pubkey(), vec![new_msg.clone()]);
         assert_eq!(
             push.process_push_message(&mut crds, &Pubkey::default(), new_msg, 0),
-            Ok(None)
+            Ok(())
         );
         assert_eq!(push.active_set.len(), 1);
         assert_eq!(push.new_push_messages(&crds, 0), expected);
@@ -759,36 +751,32 @@ mod test {
     #[test]
     fn test_personalized_push_messages() {
         let now = timestamp();
+        let mut rng = rand::thread_rng();
         let mut crds = Crds::default();
         let mut push = CrdsGossipPush::default();
-        let peer_1 = CrdsValue::new_unsigned(CrdsData::ContactInfo(ContactInfo::new_localhost(
-            &solana_sdk::pubkey::new_rand(),
-            0,
-        )));
-        assert_eq!(crds.insert(peer_1.clone(), now), Ok(None));
-        let peer_2 = CrdsValue::new_unsigned(CrdsData::ContactInfo(ContactInfo::new_localhost(
-            &solana_sdk::pubkey::new_rand(),
-            0,
-        )));
-        assert_eq!(crds.insert(peer_2.clone(), now), Ok(None));
-        let peer_3 = CrdsValue::new_unsigned(CrdsData::ContactInfo(ContactInfo::new_localhost(
-            &solana_sdk::pubkey::new_rand(),
-            now,
-        )));
+        let peers: Vec<_> = vec![0, 0, now]
+            .into_iter()
+            .map(|wallclock| {
+                let mut peer = ContactInfo::new_rand(&mut rng, /*pubkey=*/ None);
+                peer.wallclock = wallclock;
+                CrdsValue::new_unsigned(CrdsData::ContactInfo(peer))
+            })
+            .collect();
+        assert_eq!(crds.insert(peers[0].clone(), now), Ok(()));
+        assert_eq!(crds.insert(peers[1].clone(), now), Ok(()));
         assert_eq!(
-            push.process_push_message(&mut crds, &Pubkey::default(), peer_3.clone(), now),
-            Ok(None)
+            push.process_push_message(&mut crds, &Pubkey::default(), peers[2].clone(), now),
+            Ok(())
         );
         push.refresh_push_active_set(&crds, &HashMap::new(), None, &Pubkey::default(), 0, 1, 1);
 
         // push 3's contact info to 1 and 2 and 3
-        let new_msg = CrdsValue::new_unsigned(CrdsData::ContactInfo(ContactInfo::new_localhost(
-            &peer_3.pubkey(),
-            0,
-        )));
-        let mut expected = HashMap::new();
-        expected.insert(peer_1.pubkey(), vec![new_msg.clone()]);
-        expected.insert(peer_2.pubkey(), vec![new_msg]);
+        let expected: HashMap<_, _> = vec![
+            (peers[0].pubkey(), vec![peers[2].clone()]),
+            (peers[1].pubkey(), vec![peers[2].clone()]),
+        ]
+        .into_iter()
+        .collect();
         assert_eq!(push.active_set.len(), 3);
         assert_eq!(push.new_push_messages(&crds, now), expected);
     }
@@ -801,7 +789,7 @@ mod test {
             &solana_sdk::pubkey::new_rand(),
             0,
         )));
-        assert_eq!(crds.insert(peer.clone(), 0), Ok(None));
+        assert_eq!(crds.insert(peer.clone(), 0), Ok(()));
         push.refresh_push_active_set(&crds, &HashMap::new(), None, &Pubkey::default(), 0, 1, 1);
 
         let new_msg = CrdsValue::new_unsigned(CrdsData::ContactInfo(ContactInfo::new_localhost(
@@ -811,7 +799,7 @@ mod test {
         let expected = HashMap::new();
         assert_eq!(
             push.process_push_message(&mut crds, &Pubkey::default(), new_msg.clone(), 0),
-            Ok(None)
+            Ok(())
         );
         push.process_prune_msg(
             &self_id,
@@ -828,7 +816,7 @@ mod test {
             &solana_sdk::pubkey::new_rand(),
             0,
         )));
-        assert_eq!(crds.insert(peer, 0), Ok(None));
+        assert_eq!(crds.insert(peer, 0), Ok(()));
         push.refresh_push_active_set(&crds, &HashMap::new(), None, &Pubkey::default(), 0, 1, 1);
 
         let mut ci = ContactInfo::new_localhost(&solana_sdk::pubkey::new_rand(), 0);
@@ -837,7 +825,7 @@ mod test {
         let expected = HashMap::new();
         assert_eq!(
             push.process_push_message(&mut crds, &Pubkey::default(), new_msg, 1),
-            Ok(None)
+            Ok(())
         );
         assert_eq!(push.new_push_messages(&crds, 0), expected);
     }
@@ -853,9 +841,9 @@ mod test {
         // push a new message
         assert_eq!(
             push.process_push_message(&mut crds, &Pubkey::default(), value.clone(), 0),
-            Ok(None)
+            Ok(())
         );
-        assert_eq!(crds.lookup(&label), Some(&value));
+        assert_eq!(crds.get(&label).unwrap().value, value);
 
         // push it again
         assert_matches!(
