@@ -405,6 +405,7 @@ impl Validator {
             &exit,
             config.enforce_ulimit_nofile,
             &start_progress,
+            config.no_poh_speed_test,
         );
 
         *start_progress.write().unwrap() = ValidatorStartProgress::StartingServices;
@@ -635,10 +636,6 @@ impl Validator {
                 (None, None)
             };
 
-        if !config.no_poh_speed_test {
-            check_poh_speed(&genesis_config, None);
-        }
-
         let waited_for_supermajority = if let Ok(waited) = wait_for_supermajority(
             config,
             &bank,
@@ -1032,6 +1029,7 @@ fn new_banks_from_ledger(
     exit: &Arc<AtomicBool>,
     enforce_ulimit_nofile: bool,
     start_progress: &Arc<RwLock<ValidatorStartProgress>>,
+    no_poh_speed_test: bool,
 ) -> (
     GenesisConfig,
     BankForks,
@@ -1065,6 +1063,10 @@ fn new_banks_from_ledger(
         }
     }
 
+    if !no_poh_speed_test {
+        check_poh_speed(&genesis_config, None);
+    }
+
     let BlockstoreSignals {
         mut blockstore,
         ledger_signal_receiver,
